;
; TinxLib and Copy_Pic procedures...
; Copywrite  Bryan Turner 1997
;

	@library        tinxlib

FIELD_WIDTH	EQU	384		;pixels
FIELD_HEIGHT	EQU	103		;pixels

;
; Stack Frame for Copy_Pic
;
picLine		equ 0
dataRegs	equ 2+14*4		;13 data regs + return addr
outputBuff	equ dataRegs	
picBuff		equ outputBuff+4
outputWidth	equ picBuff+4
outputHeight	equ outputWidth+4
picWidth	equ outputHeight+4
picHeight	equ picWidth+4


	label Copy_Pic

DoCopyPic:
    MOVEM.L   D0-D7/A0-A3/A6,-(A7)
    SUBQ.W    #$2,A7
    MOVEA.L   picBuff(A7),A2		; A2 = picBuff
    MOVE.L    picWidth(A7),D6		; D6 = picWidth
    CLR.W     (A7)
    MOVE.L    D6,D0
    LSL.L     #$8,D0			; D0 = picWidth
    MOVE.L    outputWidth(A7),D1	; D1 = outputWidth
    DIVU.W	  D1,D0
    MOVEA.L   D0,A3			; A3 = x
    
    MOVE.L    picHeight(A7),D0		; D0 = picHeight
    LSL.L     #$8,D0
    MOVE.L    outputHeight(A7),D1	; D1 = outputHeight
    DIVU.W    D1,D0
    MOVEA.L   D0,A6			; A6 = y
    
    MOVE.L    D6,D0			; Convert picWidth in pixels to bytes...
    LSR.L     #3,D0
    AND.B     #$07,D6
    BEQ.S     IsNotOddBytes	; Non-divisible by eight numbers use an overflow byte
    ADDQ.L    #$1,D0
    
IsNotOddBytes:
    MOVE.L    D0,D6
    
    MOVE.L    outputWidth(A7),D0	; Convert outputWidth to bytes...
    MOVE.L    D0,D1
    LSR.L     #$3,D0
    AND.B     #$07,D1
    BEQ.S     NotOddBytes	; Non-divisible by eight numbers use an overflow byte
    ADDQ.L    #1,D0

NotOddBytes:
    MOVE.L    D0,outputWidth(A7)
    
    MOVEA.L   outputBuff(A7),A1
    BRA.S     CopyLineLup
    
BeginCopyLine:
    MOVE.W    (A7),D0
    EXT.L     D0
    LSR.L     #$8,D0
    MOVE.L    D6,D1
    MULU.W    D1,D0
    MOVEA.L   A2,A0				; A0 = thePicBuff
    ADDA.L    D0,A0
    MOVEQ     #$00,D3				; D3 = picIndex
    MOVE.B    #$80,D5				; D5 = bufMask
    MOVE.B    D5,D4				; D4 = picMask
    MOVE.L    outputWidth(A7),D7		; D7 = outputWidth
    BRA.S     DrawLineLup

BeginDrawLine:
    CLR.L     D0
    MOVE.B    (A0),D0				; Get a byte from the pic
    CLR.L     D1
    MOVE.B    D4,D1				; picMask->D1
    AND.L     D0,D1				; picMask & D0, is the picture pixel on?
    BEQ.S     PixNotSet				; IF yes, then turn on the buffer pixel too!

    OR.B      D5,(A1)				; Turn on pixel in buffer
    
PixNotSet:
    LSR.B     #$1,D5				; Next buffer pixel..
    BNE.S     NotNextByteBuff
    ADDQ.L    #$1,A1				; Next byte in output
    MOVE.B    #$80,D5				; Reset Mask
    SUBQ.W    #$1,D7				; One less byte in this output line..
    
NotNextByteBuff:
    MOVE.L    A3,D0				; A3 = x
    ADD.W     D0,D3				; picIndex += x

SkipPicPixels:
    MOVE.W    D3,D0
    EXT.L     D0
    LSR.L     #$8,D0
    BEQ.S     DrawLineLup			; IF ( (picIndex>>8) == 0) THEN GOTO DrawLineLup
    LSR.B     #$1,D4				; picMask>>1
    BNE.S     NotNextBytePic			; IF (picMask != 0) THEN GOTO NotNextBytePic
    MOVE.B    #$80,D4				; reset picMask
    ADDQ.L    #$1,A0				; next byte in picture...
    
NotNextBytePic:
    SUBI.W    #$0100,D3				; Subtract one increment from picIndex
    BRA.S     SkipPicPixels
    
DrawLineLup:
    TST.W     D7				; outputWidth
    BGT.S     BeginDrawLine
    MOVE.L    A6,D0				; D0 = y
    ADD.W     D0,(A7)				; picLine += y
    
CopyLineLup:
    MOVE.L    outputHeight(A7),D0
    SUBQ.L    #$1,outputHeight(A7)
    TST.L     D0
    BNE.S     BeginCopyLine
    
    ADDQ.W    #$2,A7
    MOVEM.L   (A7)+,D0-D7/A0-A3/A6
    RTS


;************ Generate Landscape *************
;* Loads a picture variable as the landscape *
;* Algorithm by: Bryan Turner                *
;* In: a5=pointer to field memory            *
;*********************************************
 label generate_landscape
	movem.l		a0-a3/d0-d3,-(a7)
	
	pea		picstr(pc)
	jsr		core[find_var]
	addq	#4,sp

	handle_ptr	d0,a0
	
	trap	#15
	clr.l	d0
	move.w	2(a0),d0			;picHeight
	move.l	D0,-(a7)
	move.w	4(a0),d0			;picWidth
	move.l	D0,-(a7)
	move.l	#FIELD_HEIGHT,-(a7)		;outputHeight
	move.l	#FIELD_WIDTH,-(a7)		;outputWidth
	pea	6(a0)				;picBuff
	move.l	a5,-(a7)			;outputBuff
	bsr		DoCopyPic
	add	#6*4,a7

	movem.l	(a7)+,a0-a3/d0-d3
	rts

picstr	dc.b	'tinxpic',0
		ds.l	0

	reloc_open
	reloc_close
	end

